/* Maximum size of operators and attribute names */
#define MAXOPLEN 256
#define MAXATTRLEN 235


/* type of service */
typedef enum ASAPkind_t {
    SERV_LOCAL, SERV_REMOTE, SERV_LOCAL_SERVER, SERV_REMOTE_SERVER
}               ASAPkind_t;

/* type of tokens on an ASAP channel */
typedef enum ASAPtoken_t {
    ASAP_BIN, ASAP_INT, ASAP_OP, ASAP_ATTROP,
    ASAP_READING_BIN, ASAP_READING_INT, ASAP_UNKNOWN
}               ASAPtoken_t;

/* predefined attributes.  "ATTR_UNKNOWN" is used as the return value of
 * "ASAPpredefAttr" when the attribute is not predefined... */
typedef enum ASAPpred_t {
    ATTR_TERMINATE, ATTR_BEGIN, ATTR_END, ATTR_INTR, ATTR_UNKNOWN
}               ASAPpred_t;

/* the "concrete" representations of the attributes */
#define ATTR_TERMINATE_NAME "@terminate"
#define ATTR_BEGIN_NAME     "@bos"
#define ATTR_END_NAME	    "@eos"
#define ATTR_INTR_NAME	    "@interrupt"

/* a type for the return values of many functions */
typedef enum ASAPstatus_t {
    ASAP_SUCCESS,		/* the call was successful */
    ASAP_FINISH,		/* call successful and finish a big data */
    ASAP_ENOSUCHSERV,		/* no such server from ASAPgetservbyname */
    ASAP_ESYNTAX,		/* syntax error in database file */
    ASAP_ENODBFILE,		/* no database file found */
    ASAP_EBADTYPE,		/* the current token is not of the expected
				 * type */
    ASAP_ESYS,			/* a system call failed. See errno for more
				 * info. */
    ASAP_ENOPORT,		/* no free port to connect */
    ASAP_BROKEN			/* connection broken */
}               ASAPstatus_t;

typedef struct ASAPserv_t {
    enum ASAPkind_t type;	/* The type of service */
    char           *cmd;	/* The command to run */
    char           *machine;	/* The remote machine */
    int             portnum;	/* The port on which the service listens */
    char           *sockname;	/* name of UNIX domain socket */
}               ASAPserv_t;

typedef struct channel_struct {
    struct in {
	int             fd;	/* file descriptor */
	char           *buf;	/* buffer */
	int             curr;	/* next to read as an index in "buf" */
	int             last;	/* last valid position in "buf" (-1 to begin) */
	int             size;	/* size of "buf" */
	ASAPtoken_t     tok;	/* next token */
	int             remaining;	/* remains to be read for current
					 * token */
    }               in;
    struct out {
	int             fd;
	char           *buf;
	int             curr;	/* next position to be written */
	int             size;	/* size of "buf" */
    }               out;
}              *channel_t;

typedef struct ASAPconn_t {
    channel_t       urgent;
    channel_t       normal;
    char           *path;	/* to retrieve the UNIX domain socket names.
				 * it is the normal one. */
}               ASAPconn_t;

/* to protect ourselves from old compilers */
#if __STDC__
#define PROTO(x) x
#else
#define PROTO(x) ()
#endif

extern char    *(*ASAPmalloc) ();
extern char    *ASAPalloc();

/* a string identifying the current version of the library */
extern char     ASAPversion[];

/* services */
extern ASAPstatus_t ASAPgetservbyname PROTO((char *name, ASAPserv_t * serv));

/* connecting */
extern ASAPstatus_t ASAPconnect PROTO((ASAPserv_t serv, ASAPconn_t * conn));
extern ASAPstatus_t ASAPlocalConnect PROTO((char *socket, ASAPconn_t * conn));
extern ASAPstatus_t ASAPremoteConnect PROTO((char *machine, int port,
					                ASAPconn_t * conn));

/* terminating */
    extern ASAPstatus_t ASAPterminate PROTO((ASAPconn_t conn));
    extern ASAPstatus_t ASAPserverEnd PROTO((ASAPconn_t conn));

/* sending */
    extern ASAPstatus_t ASAPsendNInt PROTO((channel_t chan, int i));
    extern ASAPstatus_t ASAPsendInt PROTO((channel_t chan, char *s, int len));
    extern ASAPstatus_t ASAPsendBinary PROTO((channel_t chan, char *data, int len));
    extern ASAPstatus_t ASAPsendOp PROTO((channel_t chan, char *op, int arity));
    extern ASAPstatus_t ASAPsendAttrOp PROTO((channel_t chan, char *op,
				                   int arity, int nbattrs));
    extern ASAPstatus_t ASAPsendSymbol PROTO((channel_t chan, char *symb));
    extern ASAPstatus_t ASAPsendAttrName PROTO((channel_t chan, char *attr));
    extern ASAPstatus_t ASAPflush PROTO((channel_t chan));

/* receiving */
    extern ASAPtoken_t ASAPgetTokenType PROTO((channel_t chan));
    extern ASAPstatus_t ASAPgetNInt PROTO((channel_t chan, int *i));
    extern ASAPstatus_t ASAPgetInt PROTO((channel_t chan, char *buf, int len,
					                  int *read));
    extern ASAPstatus_t ASAPgetMoreInt PROTO((channel_t chan, char *buf, int len,
					                      int *read));
    extern ASAPstatus_t ASAPgetBinary PROTO((channel_t chan, char *buf, int len,
					                     int *read));
    extern ASAPstatus_t ASAPgetMoreBinary PROTO((channel_t chan,
			                    char *buf, int len, int *read));
    extern ASAPstatus_t ASAPgetOp PROTO((channel_t chan, char *op, int *arity));
    extern ASAPstatus_t ASAPgetAttrOp PROTO((channel_t chan, char *op,
				                   int *arity, int *attrs));
    extern ASAPstatus_t ASAPgetAttrName PROTO((channel_t chan, char *attr));
    extern ASAPpred_t ASAPpredefAttr PROTO((char *attr));
    extern char    *ASAPpredefName PROTO((ASAPpred_t attr));

/* errors */
    extern char    *ASAPerrorText PROTO((ASAPstatus_t status));
    extern void    *ASAPperror PROTO((ASAPstatus_t status, char *msg));
