#ifndef MYSTRUCT_H
#define MYSTRUCT_H


/* these defines are used in reallocation for variable size array  */
/* a number of N meens that a reallocation is done each time N new */
/* elements are added to the array                                 */
/* number of sons in a tree */
#define TREESA_CHKSZ 5
/* number of stack elements */
#define STACK_CHKSZ  20
/* number of successor for a Node in an oriented graph */
#define OGNSUCC_CHKSZ 5


/* these defines are used in reallocation of variable size array              */
/* STARTSZ is the START SiZe ie the number of elements after first allocation */
/* CHKMULT is the MULTiplicator (for SiZe) applied at each reallocation (>1)  */
#define QUEUE_STARTSZ 10
#define QUEUE_MULTSZ 1.5


/* Hash Table */

typedef struct HashList {
    void           *pe;		/* pointer on a user-element */
    struct HashList *next;
}               HashList;

typedef struct HashListHeader {
    HashList       *l;
    int             lsz;
}               HashListHeader;

typedef struct HashTable {
    int             size;	/* hash table size */
    int             nbe;	/* NumBer of Elements */
    int             number;	/* unique(?) ID */
    Boolean         modified;	/* for acces optimization */
    int             lastAskedn;	/* ie */
    int             lastHTInd;	/* ie */
    int             lastEltCount;	/* ie */
    void           *(*keyFunc) ();	/* to extract key from element */
    int             (*ordFunc) ();	/* to order keys  */
    void            (*freeFunc) ();	/* to free user element */
    int             (*hashFunc) ();	/* to hash key */
    void            (*printFunc) ();	/* to print user element */
    HashListHeader *hla;	/* Hash List Array */
}               HashTable;



/* Oriented Graph */

typedef struct OGNode {
    void           *pe;		/* pointer on a user-element */
    int             flag;
    int             nbs;	/* NumBer of Successors */
    int             nbp;	/* NumBer of Predecessors */
    struct OGNode **sa;		/* Successor Array */
    struct OGraph  *pog;	/* to retrive (*funcs)() */
}               OGNode;

typedef struct OGraph {
    int             es;		/* Estiamted Size for this graph */
    int             nbn;	/* NumBer of Nodes */
    void           *(*keyFunc) ();	/* to extract key from element */
    int             (*ordFunc) ();	/* to order keys  */
    void            (*freeFunc) ();	/* to free user element */
    int             (*hashFunc) ();	/* to hash key */
    void            (*printFunc) ();	/* to print user element */
    HashTable      *ht;		/* for Node storage */
}               OGraph;


/* Tree */

typedef struct TreeNode {
    void           *pe;		/* pointer on a user-element */
    int             flag;
    int             nbs;	/* NumBer of Sons */
    struct TreeNode **sa;	/* Sons Array */
    struct Tree    *pt;		/* for easy acces to generic functions */
}               TreeNode;

typedef struct Tree {
    int             nbn;	/* NumBer of Nodes */
    void           *(*keyFunc) ();	/* to extract key from element */
    int             (*ordFunc) ();	/* to order keys  */
    void            (*freeFunc) ();	/* to free user element */
    void            (*printFunc) ();	/* to print user element */
    TreeNode       *root;	/* tree root */
}               Tree;


/* List */

typedef struct ListElt {
    void           *pe;
    int             flag;
    struct ListElt *next;
}               ListElt;

typedef struct List {
    int             nbe;	/* NumBer of Elements */
    int             number;	/* unique(?) ID */
    Boolean         modified;	/* for acces optimization */
    int             lastN;	/* ie */
    ListElt        *lastAns;	/* ie */
    void           *(*keyFunc) ();	/* to extract key from element */
    int             (*ordFunc) ();	/* to order keys  */
    void            (*freeFunc) ();	/* to free user element */
    void            (*printFunc) ();	/* to print user element */
    ListElt        *first;	/* list first head */
}               List;


/* Stack */

typedef struct Stack {
    int             nbe;	/* NumBer of Elements */
    int             sz;		/* Stack Size */
    void          **ea;		/* Element Array */
}               Stack;


/* Queue */

typedef struct Queue {
    int             nbe;	/* number of element of this queue */
    int             sz;		/* actual maximum size  " */
    int             ihead;	/* the oldest element of the queue */
    int             iend;	/* the newest " */
    int             firstNull;	/* the first Null element (cf QNull()) */
    void          **ea;		/* the queue (warning cyclic storage) */
    /* growing form left to right */
}               Queue;


/* List of Strings */

typedef struct LString {
    char           *s;
    struct LString *next;
}               LString;




/**********/
/* protos */

int             hashString(char *str, int sz);
int             hashInt(int *i, int sz);
int             hashVoid(void *p, int sz);
int             ordString(char *s1, char *s2);
int             ordInt(int *a, int *b);
int             ordVoid(void *a, void *b);
void            printString(char *pe);
void            printInt(int *pe);
void            printVoid(void *pe);
void           *keySame(void *pe);
void           *keyVoid(void *pe);

HashTable      *htCreate(int size, void *(*keyFunc) (), int (*orderFunc) (), void (*freeFunc) (), int (*hashFunc) (), void (*printFunc) ());
void            htFree(HashTable * pht, unsigned char freeElt);
void           *htSearch(HashTable * pht, void *pe);
void           *htSearchKey(HashTable * pht, void *key);
void            htInsert(HashTable * pht, void *pe);
void            htDel(HashTable * pht, void *pe, unsigned char freeElt);
int             htNbElts(HashTable * pht);

void           *htFindNth(HashTable * pht, int n);
OGraph         *ogCreate(int orderOfSize, void *(*keyFunc) (), int (*orderFunc) (), void (*freeFunc) (), int (*hashFunc) (), void (*printFunc) ());
void            ogFree(OGraph * pog);
void            ogAddNode(OGraph * pog, void *pe);
void            ogAddArc(OGraph * pog, void *pa, void *pb);
Boolean         ogCheckCycle(OGraph * pog);
List           *ogFindOrder(OGraph * pog);
Tree           *ogFindTree(OGraph * pog);

Tree           *tCreate(void *(*keyFunc) (), int (*orderFunc) (), void (*freeFunc) (), void (*printFunc) ());
void            tFree(Tree * pt, unsigned char freeElt);
TreeNode       *tAddNode(Tree * pt, TreeNode * ptnf, void *pe);
int             tNbSons(TreeNode * ptn);
List           *tBreadthFirst(Tree * pt);
List           *tDephtFirst(Tree * pt);
void            tPrint(Tree * pt);

List           *lCreate(void *(*keyFunc) (), int (*orderFunc) (), void (*freeFunc) (), void (*printFunc) ());
void            lFree(List * pl, unsigned char freeElt);
void            lAdd(List * pl, void *pe);
void            lAddUnique(List * pl, void *pe);
int             lNbElts(List * pl);
void           *lFindNth(List * pl, int n);
void            lDel(List * pl, void *pe, unsigned char freeElt);
void            lDelAll(List * pl, void *pe, unsigned char freeElt);
Boolean         lIn(List * pl, void *pe);
void           *lSearch(List * pl, void *pe);
void            lReverse(List * pl);
void            lPrint(List * pl);

Stack          *sCreate(void);
void            sFree(Stack * ps);
Boolean         sIsEmpty(Stack * ps);
void           *sPop(Stack * ps);
void            sPush(Stack * ps, void *pe);
void           *sLook(Stack * ps);


/* warning: the first element of a queue is numbered 1 (and not 0) */
Queue          *qCreate(void);
void            qFree(Queue * pq);
void            qAdd(Queue * pq, void *pe);
void           *qGet(Queue * pq);
void           *qLook(Queue * pq);
int             qNbElts(Queue * pq);
void           *qLookNb(Queue * pq, int nb);
void            qDel(Queue * pq, int nb, void (*freeFunc) ());
void           *qShift(Queue * pq, int start, int shift);
void            qNullNb(Queue * pq, int nb);
void            qCompactNull(Queue * pq);

LString        *lsAdd(LString * ls, char *s);
Boolean         lsIn(LString * ls, char *s);
void           *lLookKey ( List * pl, void *pk );
void            lsFree(LString * ls, unsigned char freeElt);


#endif
