#include <stdio.h>
#include <X11/Xlib.h>
#include <values.h>
#include <malloc.h>
#include <stdlib.h>

#include "myAll.h"

#define NB_BEZIER_STEPS 50


int             StXRGBDist(XColor * c1, XColor * c2)
{
    return abs(c1->red - c2->red) + abs(c1->green - c2->green) + abs(c1->blue - c2->blue);
}


GC              StXInvertGc(Display * dpy, Drawable d, GC srcGc)
{
    GC              invGc;
    XGCValues       gcv;
    unsigned long   tmpground;

    XGetGCValues(dpy, srcGc, (GCForeground | GCBackground), &gcv);
    tmpground = gcv.foreground;
    gcv.foreground = gcv.background;
    gcv.background = tmpground;
    invGc = XCreateGC(dpy, d, (GCForeground | GCBackground), &gcv);
    return invGc;
}




int             StXFindNearestColor(Display * dpy, XColor * colRef, int *dist)
{
    Colormap        defcm;
    XColor         *qcols;
    Screen         *screen;
    int             nbCells;
    int             bestDist, tmpd;
    int             bestCol;
    int             i;

    screen = ScreenOfDisplay(dpy, DefaultScreen(dpy));
    defcm = DefaultColormapOfScreen(screen);
    nbCells = CellsOfScreen(screen);
    qcols = (XColor *) malloc(nbCells * sizeof(XColor));
    for (i = 0; i < nbCells; i++) {
	qcols[i].pixel = i;
    }
    XQueryColors(dpy, defcm, qcols, nbCells);
    bestDist = MAXINT;
    for (i = 0; i < nbCells; i++) {
	tmpd = StXRGBDist(colRef, &qcols[i]);
	if (tmpd < bestDist) {
	    bestDist = tmpd;
	    bestCol = i;
	}
    }
    *dist = bestDist;
    return bestCol;
}


void            StXReduce4Ximage(Display * dpy, XImage * srci, XImage * desti, int ulx, int uly, int w, int h, int *greyMap)
{
    register int    val, sx, sy, dx, dy;
    int             black;

    black = greyMap[4];
    for (dx = 0; dx < (w >> 1); dx++) {
	sx = dx << 1;
	for (dy = 0; dy < (h >> 1); dy++) {
	    sy = dy << 1;
	    val = 0;
	    if (XGetPixel(srci, sx, sy) == black)
		val++;
	    sx++;
	    if (XGetPixel(srci, sx, sy) == black)
		val++;
	    sy++;
	    if (XGetPixel(srci, sx, sy) == black)
		val++;
	    sx--;
	    if (XGetPixel(srci, sx, sy) == black)
		val++;

	    XPutPixel(desti, dx, dy, greyMap[val]);
	}
    }
}


void            StXDrawBezier(Display * dpy, Drawable d, GC gc, int x0, int y0, int x1, int y1, int x2, int y2, int x3, int y3)
{
    float           dt, ax, bx, cx, ay, by, cy, t = 0.0;
    int             nsteps = NB_BEZIER_STEPS;
    int             px1, py1, px2, py2;
    char            first = 1;

    dt = 1.0 / nsteps;
    cx = 3.0 * (x1 - x0);
    bx = 3.0 * (x2 - x1) - cx;
    ax = x3 - (x0 + cx + bx);
    cy = 3.0 * (y1 - y0);
    by = 3.0 * (y2 - y1) - cy;
    ay = y3 - (y0 + cy + by);

    while (t <= 1.0 + dt / 10.0) {
	px1 = px2;
	py1 = py2;
	px2 = ((ax * t + bx) * t + cx) * t + x0;
	py2 = ((ay * t + by) * t + cy) * t + y0;
	if (!first) {
	    XDrawLine(dpy, d, gc, px1, py1, px2, py2);
	}
	else {
	    first = 0;
	}
	t += dt;
    }
}
