#ifndef AGATXLABEL_H
#define AGATXLABEL_H

/* find the ideal(?) value to use for ticks drawing                  */
/* use Paul Heckbert 'nicenum' algorithm                             */
/* min,max : ticks desired extrema,                                  */
/* warning these values are probably not used 'as is'                */
/* returned min and max (found in aTicks) will probably              */
/* be slightly different (but covering at least the                  */
/* same range and probably in a nicer way)                           */
/* nbTicks : amount of desired ticks. Reevaluate                     */
/* aTicks  : array to store computed ticks (must be allocated        */
/* before call. Realloc if nbTicks reevaluate)                       */
void            findTicks(double min, double max, int *nbTicks, double **aTicks);



/* use aTicks to fill in aLabs with strings to print as ticks labels */
/* nbTicks : amount of desired ticks                                 */
/* nbcar   : maximum length of string to build                       */
/* aTicks  : precomputed array of ticks values (see findTicks)       */
/* aLabs   : array to fill with labels (must be preallocated)        */
void            findLabels(int nbTicks, int nbcar, double *aTicks, char **aLabs);



/* Build an array of doubles and of strings and return the number of Ticks to display on X
 * xsz         : size of the window
 * fontWidth   : the width of the font to display
 * minx, maxx  : minimum and maximum value to display
 * hRatio      : what part of the screen must be coverred by X labels
 * aTicks      : array of double which is build
 * aLabs       : array of string which is build
 */
int buildXTicksLabs ( int xsz, int fontWidth, double minx, double maxx, 
                      double hRatio, double **aTicks, char ***aLabs );



/* Build an array of doubles and of strings and return the number of Ticks to display on Y
 * xsz, ysz              : size of the window
 * fontHeight, fontWidth : the width of the font to display
 * miny, maxy            : minimum and maximum value to display
 * nbCarMin              : no y label with less then this lenght
 * vRatio, hRatio        : what part of the screen must be coverred by Y labels
 *                         vRatio == 5 -> 1/5 of screen height is used for label print 
 *                         hRatio == 8 -> 1/8 of screen widht  is used for label print 
 * nbLabCar              : return val for number of char of a label
 * aTicks                : array of double to build
 * aLabs                 : array of string to build
 */
int buildYTicksLabs(int xsz, int ysz, int fontHeight, int fontWidth, 
                    double miny, double maxy, int nbCarMin, int vRatio, int hRatio, 
                    int *nbLabCar, double **aTicks, char ***aLabs);



/* example: */
#ifdef NEVER_DEFINED
printTicks(double min, double max, int nbTicks, int nbcar)
{
    int             i;
    double         *aTicks;
    char          **aLabs;

    /* allocate aLabs in a non brute way :) only one block pointers + strings */
    aLabs = (char **) myMalloc(nbTicks * (sizeof(char *) + (nbcar + 1)));
    for (i = 0; i < nbTicks; i++)
	aLabs[i] = (char *) (aLabs + nbTicks) + i * (nbcar + 1);
    /* allocate aTicks */
    aTicks = (double *) myMalloc(sizeof(double) * nbTicks);

    findTicks(min, max, nbTicks, aTicks);
    findLabels(nbTicks, nbcar, aTicks, aLabs);
    for (i = 0; i < nbTicks; i++)
	printf("%s\n", aLabs[i]);

    free(aLabs);
    free(aTicks);
}

#endif				/* NEVER_DEFINED */

#endif
